searchData={"items":[{"type":"module","title":"S3","doc":"Small S3-compatible API request builder and stuff","ref":"S3.html"},{"type":"function","title":"S3.build/1","doc":"Builds URI, headers, body triplet to be used with HTTP clients.","ref":"S3.html#build/1"},{"type":"function","title":"S3.sign/1","doc":"Returns a presigned URL","ref":"S3.html#sign/1"},{"type":"function","title":"S3.signature/1","doc":"Calculates V4 signature","ref":"S3.html#signature/1"},{"type":"function","title":"S3.xml/1","doc":"Decodes XML binaries and encodes term to XML iodata.\n\nExamples:\n\n    iex> xml(\"\")\n    ** (ArgumentError) Can't detect character encoding due to lack of indata\n\n    iex> xml(\"  \")\n    {:ok, {\"Hello\", []}}\n\n    iex> IO.iodata_to_binary(xml({\"Hello\", []}))\n    \"  \"\n\n    iex> xml(\"\"\"\n    ...>  \n    ...>\n    ...>   Learning Amazon Web Services  \n    ...>\n    ...>   Mark Wilkins  \n    ...>\n    ...>  \n    ...> \"\"\")\n    {:ok, {\"book\", [{\"title\", [\" Learning Amazon Web Services \"]}, {\"author\", [\" Mark Wilkins \"]}]}}\n\n    iex> IO.iodata_to_binary(xml({\"book\", [{\"title\", [\" Learning Amazon Web Services \"]}, {\"author\", [\" Mark Wilkins \"]}]}))\n    \"   Learning Amazon Web Services    Mark Wilkins   \"\n\n    iex> xml(\"\"\"\n    ...> <?xml version=\"1.0\" encoding=\"UTF-8\"?>\n    ...> <俄语 լեզու=\"ռուսերեն\">данные</俄语>\n    ...> \"\"\")\n    {:ok, {\"俄语\", [\"данные\"]}}\n\n    iex> IO.iodata_to_binary(xml({\"俄语\", [\"данные\"]}))\n    \"<俄语>данные</俄语>\"","ref":"S3.html#xml/1"},{"type":"type","title":"S3.headers/0","doc":"","ref":"S3.html#t:headers/0"},{"type":"type","title":"S3.option/0","doc":"","ref":"S3.html#t:option/0"},{"type":"type","title":"S3.xml_element/0","doc":"","ref":"S3.html#t:xml_element/0"},{"type":"extras","title":"README","doc":"Small and experimental Amazon S3-compatible object storage request builder.\n\nVerified to work with Amazon S3, MinIO.\n\nTODO: Wasabi, Backblaze B2, Cloudflare R2, DigitalOcean, and Scaleway.\n\n##","ref":"readme.html"},{"type":"extras","title":"Example using [MinIO](https://github.com/minio/minio) and [Finch](https://github.com/sneako/finch) - README","doc":"```console\n// don't forget to `docker stop minio` once done\n$ docker run -d --rm -p 9000:9000 --name minio minio/minio server /data\n$ docker exec minio mc alias set local http://localhost:9000 minioadmin minioadmin\n$ docker exec minio mc mb local/testbucket\n$ iex\n```\n\n```elixir\n# Setup\nMix.install([:finch, :s3])\nFinch.start_link(name: MinIO.Finch)\n\nconfig = fn options ->\n  Keyword.merge(\n    [\n      access_key_id: \"minioadmin\",\n      secret_access_key: \"minioadmin\",\n      url: \"http://localhost:9000\",\n      region: \"us-east-1\"\n    ],\n    options\n  )\nend\n```\n\n```elixir\n# PutObject\n{uri, headers, body} =\n  S3.build(\n    config.(\n      method: :put,\n      headers: [{\"content-type\", \"application/octet-stream\"}],\n      path: \"/testbucket/my-bytes\",\n      body: <<0::size(8 * 1_000_000)>>\n    )\n  )\n\nreq = Finch.build(:put, uri, headers, body)\n200 = Finch.request!(req, MinIO.Finch).status\n```\n\n```elixir\n# HeadObject\n{uri, headers, body} = S3.build(config.(method: :head, path: \"/testbucket/my-bytes\"))\nreq = Finch.build(:head, uri, headers, body)\n\n%{\n  \"content-length\" => \"1000000\",\n  \"content-type\" => \"application/octet-stream\",\n  \"etag\" => \"\\\"879f4bba57ed37c9ec5e5aedf9864698\\\"\"\n  # etc.\n} = Map.new(Finch.request!(req, MinIO.Finch).headers)\n```\n\n```elixir\n# stream GetObject\n{uri, headers, body} = S3.build(config.(method: :get, path: \"/testbucket/my-bytes\"))\nreq = Finch.build(:get, uri, headers, body)\n\nstream = fn packet, _acc ->\n  with {:data, data} <- packet do\n    IO.inspect(byte_size(data), label: \"bytes received\")\n  end\nend\n\nFinch.stream(req, MinIO.Finch, _acc = [], stream)\n# bytes received: 147404\n# bytes received: 408300\n# bytes received: 408300\n# bytes received: 35996\n```\n\n```elixir\n# chunked PutObject\n# https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-streaming.html\nstream = Stream.repeatedly(fn -> <<0::size(8 * 100_000)>> end)\nstream = Stream.take(stream, 10)\n\n{uri, headers, body = {:stream, _signed_stream}} =\n  S3.build(\n    config.(\n      method: :put,\n      headers: [\n        {\"content-type\", \"application/octet-stream\"},\n        {\"content-encoding\", \"aws-chunked\"},\n        {\"x-amz-decoded-content-length\", \"1000000\"}\n      ],\n      path: \"/testbucket/my-bytestream\",\n      body: {:stream, stream}\n    )\n  )\n\nreq = Finch.build(:put, uri, headers, body)\n200 = Finch.request!(req, MinIO.Finch).status\n```\n\n```elixir\n# ListObjectsV2\n# https://docs.aws.amazon.com/AmazonS3/latest/API/API_ListObjectsV2.html\n{uri, headers, body} = S3.build(config.(method: :get, path: \"/testbucket\", query: %{\"list-type\" => 2}))\nreq = Finch.build(:get, uri, headers, body)\n\n{:ok,\n {\n   \"ListBucketResult\",\n   [\n     {\"Name\", [\"testbucket\"]},\n     {\"Prefix\", []},\n     {\"KeyCount\", [\"2\"]},\n     {\"MaxKeys\", [\"1000\"]},\n     {\"IsTruncated\", [\"false\"]}\n     | contents = [\n         {\n           \"Contents\",\n           [\n             {\"Key\", [\"my-bytes\"]},\n             {\"LastModified\", [\"2023-12-14T08:54:40.085Z\"]},\n             {\"ETag\", [\"\\\"879f4bba57ed37c9ec5e5aedf9864698\\\"\"]},\n             {\"Size\", [\"1000000\"]},\n             {\"StorageClass\", [\"STANDARD\"]}\n           ]\n         }\n         | _etc\n       ]\n   ]\n }} =\n  S3.xml(Finch.request!(req, MinIO.Finch).body)\n\n# DeleteObjects\n# https://docs.aws.amazon.com/AmazonS3/latest/API/API_DeleteObjects.html\nobjects =\n  Enum.map(contents, fn {\"Contents\", contents} ->\n    {\"Object\", [List.keyfind!(contents, \"Key\", 0)]}\n  end)\n\nxml = S3.xml({\"Delete\", objects})\ncontent_md5 = Base.encode64(:crypto.hash(:md5, xml))\n\n{uri, headers, body} =\n  S3.build(\n    config.(\n      method: :post,\n      path: \"/testbucket\",\n      query: %{\"delete\" => \"\"},\n      headers: [{\"content-md5\", content_md5}],\n      body: xml\n    )\n  )\n\n{:ok,\n {\"DeleteResult\",\n  [\n    {\"Deleted\", [{\"Key\", [\"my-bytes\"]}]}\n  ]\n  | _etc}} =\n  S3.xml(Finch.request!(req, MinIO.Finch).body)\n```\n\n```elixir\n# Signed Upload Form\n# https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-UsingHTTPPOST.html\n# similar to https://gist.github.com/chrismccord/37862f1f8b1f5148644b75d20d1cb073\n\n# TODO what the api should be? S3.form/1\n```\n\n```elixir\n# Signed URL\n# https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html\n\n%URI{} = url = \n  S3.url(\n    access_key_id: \"AKIAIOSFODNN7EXAMPLE\",\n    secret_access_key: \"wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY\",\n    region: \"us-east-1\",\n    method: :get,\n    url: \"https://examplebucket.s3.amazonaws.com\",\n    path: \"/test.txt\",\n    query: %{\"X-Amz-Expires\" => 86400},\n    utc_now: ~U[2013-05-24 00:00:00Z]\n  )\n\n\"\"\"\nhttps://examplebucket.s3.amazonaws.com/test.txt?\\\nX-Amz-Algorithm=AWS4-HMAC-SHA256&\\\nX-Amz-Credential=AKIAIOSFODNN7EXAMPLE%2F20130524%2Fus-east-1%2Fs3%2Faws4_request&\\\nX-Amz-Date=20130524T000000Z&\\\nX-Amz-Expires=86400&\\\nX-Amz-SignedHeaders=host&\\\nX-Amz-Signature=aeeed9bbccd4d02ee5c0109b86d86835f995330da4c265957d157751f604d404\\\n\"\"\" = URI.to_string(url)\n```","ref":"readme.html#example-using-minio-https-github-com-minio-minio-and-finch-https-github-com-sneako-finch"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.1.0 (2024-03-31) - Changelog","doc":"- cleanup, upadte deps, add some docs","ref":"changelog.html#0-1-0-2024-03-31"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}